#region Using Statements
using System;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Input;

#endregion

namespace SpaceShooter
{
    // =======================================================================
    // Game1, detta är vår huvudklass i spelet, det är här all action sker!
    // =======================================================================
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics; // Används för att hantera grafik
        SpriteBatch spriteBatch; // Används för att rita bilder
        Texture2D ship_texture; // Rymdskeppets textur
        Vector2 ship_vector; // Rymdskeppets koordinater
        Vector2 ship_speed; // Rymdskeppets hastighet

        // =======================================================================
        // Game1(), klassens konstruktor
        // =======================================================================
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        // =======================================================================
        // Initialize(), anropas då spelet startar. Här ligger all kod för att 
        // initiera objekt och skapa demdock inte laddning av olika filer (bilder,
        // ljud etc).
        // =======================================================================
        protected override void Initialize()
        {
            ship_vector.X = 380;
            ship_vector.Y = 400;
            ship_speed.X = 2.5f;
            ship_speed.Y = 4.5f;
            base.Initialize();
        }

        // =======================================================================
        // LoadContent(), anropas då spelet startar. Här laddas alla objekt/filer
        // in (bilder, ljud etc)
        // =======================================================================
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            ship_texture = Content.Load<Texture2D>("images/player/ship");
        }

        // =======================================================================
        // UnloadContent(), anropas då spelet avslutas. Här kan man ladda ur de
        // objekt som skulle kunna behöva det för att rensa minne.
        // =======================================================================
        protected override void UnloadContent()
        {

        }

        // =======================================================================
        // Update(), första delen av spel-loopen. Här föränras olika objekt. T.ex.
        // så läser vi in data från användaren, flyttar på olika objekt osv.
        // gameTime, används för att hålla koll på spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Update(GameTime gameTime)
        {
            // Stänger av spelet om man trycker på Back-knappen på gamepaden.
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back ==
                                                              ButtonState.Pressed)
                this.Exit();

            // Läs in tangenttryckningar:
            KeyboardState keyboardState = Keyboard.GetState();

            // Flytta rymdskeppet efter tangenttryckningar (om det inte är på
            // väg ut från kanten):
            if (ship_vector.X <= Window.ClientBounds.Width - ship_texture.Width
                && ship_vector.X >= 0)
            {
                if (keyboardState.IsKeyDown(Keys.Right))
                    ship_vector.X += ship_speed.X;
                if (keyboardState.IsKeyDown(Keys.Left))
                    ship_vector.X -= ship_speed.X;
            }
            if (ship_vector.Y <= Window.ClientBounds.Height - ship_texture.Height
                && ship_vector.Y >= 0)
            {
                if (keyboardState.IsKeyDown(Keys.Down))
                    ship_vector.Y += ship_speed.Y;
                if (keyboardState.IsKeyDown(Keys.Up))
                    ship_vector.Y -= ship_speed.Y;
            }

            // Kontrollera ifall rymdskeppet har åkt  ut från kanten, om det har
            // det, så återställ dess position.
            // Har det åkt ut till vänster:
            if (ship_vector.X < 0)
                ship_vector.X = 0;
            // Har det åkt ut till höger:
            if (ship_vector.X > Window.ClientBounds.Width - ship_texture.Width)
                ship_vector.X = Window.ClientBounds.Width - ship_texture.Width;
            // Har det åkt ut upptill:
            if (ship_vector.Y < 0)
                ship_vector.Y = 0;
            // Har det åkt ut nedtill:
            if (ship_vector.Y > Window.ClientBounds.Height - ship_texture.Height)
                ship_vector.Y = Window.ClientBounds.Height - ship_texture.Height;

            base.Update(gameTime);
        }

        // =======================================================================
        // Draw(), Här ritas själva spelet ut.
        // gameTime, används för att hålla koll på spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // Rensa skärmen

            // Använd spriteBatch för att rita ut saker på skärmen
            spriteBatch.Begin();
            spriteBatch.Draw(ship_texture, ship_vector, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}

