using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Input;

namespace SpaceShooter
{
    // =======================================================================
    // Game1, detta är vår huvudklass i spelet, det är här all action sker!
    // =======================================================================
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics; // Används för att hantera grafik
        SpriteBatch spriteBatch; // Används för att rita bilder
        Player player; // Spelarens objekt
        List<Enemy> enemies; // Lista med fiender

        // =======================================================================
        // Game1(), klassens konstruktor
        // =======================================================================
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        // =======================================================================
        // Initialize(), anropas då spelet startar. Här ligger all kod för att 
        // initiera objekt och skapa demdock inte laddning av olika filer (bilder,
        // ljud etc).
        // =======================================================================
        protected override void Initialize()
        {

            base.Initialize();
        }

        // =======================================================================
        // LoadContent(), anropas då spelet startar. Här laddas alla objekt/filer
        // in (bilder, ljud etc)
        // =======================================================================
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Skapa spelaren:
            player = new Player(Content.Load<Texture2D>("images/player/ship"), 380, 400, 2.5f, 4.5f);
            
            // Skapa fiender
            enemies = new List<Enemy>();
            Random random = new Random();
            Texture2D tmpSprite = Content.Load<Texture2D>("images/enemies/mine");
            for (int i = 0; i < 10; i++)
            {
                int rndX = random.Next(0, Window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, Window.ClientBounds.Height / 2);
                Enemy temp = new Enemy(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lägg till i listan
            }

        }

        // =======================================================================
        // UnloadContent(), anropas då spelet avslutas. Här kan man ladda ur de
        // objekt som skulle kunna behöva det för att rensa minne.
        // =======================================================================
        protected override void UnloadContent()
        {

        }

        // =======================================================================
        // Update(), första delen av spel-loopen. Här föränras olika objekt. T.ex.
        // så läser vi in data från användaren, flyttar på olika objekt osv.
        // gameTime, används för att hålla koll på spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Update(GameTime gameTime)
        {
            // Stänger av spelet om man trycker på Back-knappen på gamepaden.
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back ==
                                                              ButtonState.Pressed)
                this.Exit();

            player.Update(Window); // Uppdatera spelarens position

            // Gå igenom alla fiender
            foreach (Enemy e in enemies.ToList())
            {
                if (e.IsAlive) // Kontrollera om fienden lever
                    e.Update(Window); // Flytta på dem
                else // Ta bort fienden för den är död
                    enemies.Remove(e);
            }

            base.Update(gameTime);
        }

        // =======================================================================
        // Draw(), Här ritas själva spelet ut.
        // gameTime, används för att hålla koll på spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // Rensa skärmen

            // Använd spriteBatch för att rita ut saker på skärmen
            spriteBatch.Begin();
            
            player.Draw(spriteBatch); // Rita ut spelaren

            // Rita ut alla fiender:
            foreach (Enemy e in enemies)
                e.Draw(spriteBatch);
            
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}