﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SpaceShooter
{
    // =======================================================================
    // BackgroundSprite, behållareklass för en bakgrundsbild. Denna typ av
    // bakgrundsbild är en del av en 2dvektor med flera BackgroundSprite-
    // objekt.
    // =======================================================================
    class BackgroundSprite : GameObject
    {
        // =======================================================================
        // BackgroundSprite(), konstruktor för att skapa BackgroundSprite-objekt
        // =======================================================================
        public BackgroundSprite(Texture2D texture, float X, float Y) :
            base(texture, X, Y)
        {
        }

        // =======================================================================
        // Update(), ändrar positionen för ett BackgroundSprite-objekt. Flyttar
        // det längst upp ifall det har gåt ut i nedkant av skärmen.
        // =======================================================================
        public void Update(GameWindow window, int nrBackgroundsY)
        {
            vector.Y += 2f; // Flytta bakgrunden
            // Kontrollera om bakgrunden har åkt ut i nedkant:
            if (vector.Y > window.ClientBounds.Height)
            {
                // Flytta bilden så att den hamnar ovanför alla andra
                // bagkrundsbilder:
                vector.Y = vector.Y - nrBackgroundsY * texture.Height;
            }
        }
    }
    // =======================================================================
    // Background, klass för att rita ut en 2d-vektor med bakgrundsbilder.
    // =======================================================================
    class Background
    {
        BackgroundSprite[,] background;
        int nrBackgroundsX, nrBackgroundsY;

        // =======================================================================
        // Background(), konstruktor som skapar alla BackgroundSprite-objekt i en
        // tvådimensionell vektor.
        // =======================================================================
        public Background(Texture2D texture, GameWindow window)
        {
            // Hur många bilder ska vi ha på bredden?
            double tmpX = (double)window.ClientBounds.Width / texture.Width;
            // Avrunda uppåt med Math.Ceiling():
            nrBackgroundsX = (int)Math.Ceiling(tmpX);
            // Hur många bilder ska vi ha på höjden?
            double tmpY = (double)window.ClientBounds.Height / texture.Height;
            // Avrunda uppåt med Math.Ceiling(), lägg till 1 extra:
            nrBackgroundsY = (int)Math.Ceiling(tmpY) + 1;

            // Sätt storlek på vektorn:
            background = new BackgroundSprite[nrBackgroundsX, nrBackgroundsY];

            // Fyll på vektorn med BackgroundSprite-objekt:
            for (int i = 0; i < nrBackgroundsX; i++)
            {
                for (int j = 0; j < nrBackgroundsY; j++)
                {
                    int posX = i * texture.Width;
                    // Gör att den första hamnar ovanför skärmen:
                    int posY = j * texture.Height - texture.Height;
                    background[i, j] = new BackgroundSprite(texture, posX, posY);
                }
            }
        }


        // =======================================================================
        // Update(), uppdaterar positionen för samtliga BackgoundSprite-objekt.
        // =======================================================================
        public void Update(GameWindow window)
        {
            for (int i = 0; i < nrBackgroundsX; i++)
                for (int j = 0; j < nrBackgroundsY; j++)
                    background[i, j].Update(window, nrBackgroundsY);
        }


        // =======================================================================
        // Draw(), ritar ut samtliga BackgoundSprite-objekt.
        // =======================================================================
        public void Draw(SpriteBatch spriteBatch)
        {
            for (int i = 0; i < nrBackgroundsX; i++)
                for (int j = 0; j < nrBackgroundsY; j++)
                    background[i, j].Draw(spriteBatch);
        }

    }

}