using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace SpaceShooter
{
    // =======================================================================
    // Game1, detta är vår huvudklass i spelet.
    // =======================================================================
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics; // Används för att hantera grafik
        SpriteBatch spriteBatch; // Används för att rita bilder
        // =======================================================================
        // Game1(), klassens konstruktor
        // =======================================================================
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        // =======================================================================
        // Initialize(), anropas då spelet startar. Här ligger all kod för att 
        // initiera objekt och skapa dem dock inte laddning av olika filer
        // (bilder, ljud etc).
        // =======================================================================
        protected override void Initialize()
        {
            GameElements.currentState = GameElements.State.Menu;
            GameElements.Initialize();
            base.Initialize();
        }

        // =======================================================================
        // LoadContent(), anropas då spelet startar. Här laddas alla objekt/filer
        // in (bilder, ljud etc)
        // =======================================================================
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            GameElements.LoadContent(Content, Window);
        }

        // =======================================================================
        // UnloadContent(), anropas då spelet avslutas. Här kan man ladda ur de
        // objekt som skulle kunna behöva det för att rensa minne.
        // =======================================================================
        protected override void UnloadContent()
        {

        }

        // =======================================================================
        // Update(), första delen av spel-loopen. Här föränras olika objekt. T.ex.
        // så läser vi in data från användaren, flyttar på olika objekt osv.
        // gameTime, används för att hålla koll på spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Update(GameTime gameTime)
        {
            // Stänger av spelet om man trycker på Back-knappen på gamepaden.
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            switch (GameElements.currentState)
            {
                case GameElements.State.Run: // Kör själva spelet
                    GameElements.currentState = GameElements.RunUpdate(Content, Window,
                                                                gameTime);
                    break;
                case GameElements.State.HighScore: // Highscorelistan
                    GameElements.currentState = GameElements.HighScoreUpdate();
                    break;
                case GameElements.State.Quit: // Avsluta spelet
                    this.Exit();
                    break;
                default: // Menyn
                    GameElements.currentState = GameElements.MenuUpdate(gameTime);
                    break;
            }

            base.Update(gameTime);
        }

        // =======================================================================
        // Draw(), Här ritas själva spelet ut.
        // gameTime, används för att hålla koll på spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // Rensa skärmen

            // Använd spriteBatch för att rita ut saker på skärmen
            spriteBatch.Begin();

            switch (GameElements.currentState)
            {
                case GameElements.State.Run: // Kör själva spelet
                    GameElements.RunDraw(spriteBatch);
                    break;
                case GameElements.State.HighScore: // Highscorelistan
                    GameElements.HighScoreDraw(spriteBatch);
                    break;
                case GameElements.State.Quit: // Avsluta spelet
                    this.Exit();
                    break;
                default: // Menyn
                    GameElements.MenuDraw(spriteBatch);
                    break;
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }

}