using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace SpaceShooter
{
    // =======================================================================
    // Game1, detta r vr huvudklass i spelet, det r hr all action sker!
    // =======================================================================
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics; // Anvnds fr att hantera grafik
        SpriteBatch spriteBatch; // Anvnds fr att rita bilder
        Texture2D ship_texture; // Rymdskeppets textur
        Vector2 ship_vector; // Rymdskeppets koordinater
        Vector2 ship_speed; // Rymdskeppets hastighet

        // =======================================================================
        // Game1(), klassens konstruktor
        // =======================================================================
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        // =======================================================================
        // Initialize(), anropas d spelet startar. Hr ligger all kod fr att 
        // initiera objekt och skapa demdock inte laddning av olika filer (bilder,
        // ljud etc).
        // =======================================================================
        protected override void Initialize()
        {
            ship_vector.X = 380;
            ship_vector.Y = 400;
            ship_speed.X = 2.5f;
            ship_speed.Y = 4.5f;
            base.Initialize();
        }

        // =======================================================================
        // LoadContent(), anropas d spelet startar. Hr laddas alla objekt/filer
        // in (bilder, ljud etc)
        // =======================================================================
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            ship_texture = Content.Load<Texture2D>("images/player/ship");
        }

        // =======================================================================
        // UnloadContent(), anropas d spelet avslutas. Hr kan man ladda ur de
        // objekt som skulle kunna behva det fr att rensa minne.
        // =======================================================================
        protected override void UnloadContent()
        {

        }

        // =======================================================================
        // Update(), frsta delen av spel-loopen. Hr frnras olika objekt. T.ex.
        // s lser vi in data frn anvndaren, flyttar p olika objekt osv.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Update(GameTime gameTime)
        {
            // Stnger av spelet om man trycker p Back-knappen p gamepaden.
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back ==
                                                              ButtonState.Pressed)
                this.Exit();

            // Ls in tangenttryckningar:
            KeyboardState keyboardState = Keyboard.GetState();

            // Flytta rymdskeppet efter tangenttryckningar (om det inte r p
            // vg ut frn kanten):
            if (ship_vector.X <= Window.ClientBounds.Width - ship_texture.Width
                && ship_vector.X >= 0)
            {
                if (keyboardState.IsKeyDown(Keys.Right))
                    ship_vector.X += ship_speed.X;
                if (keyboardState.IsKeyDown(Keys.Left))
                    ship_vector.X -= ship_speed.X;
            }
            if (ship_vector.Y <= Window.ClientBounds.Height - ship_texture.Height
                && ship_vector.Y >= 0)
            {
                if (keyboardState.IsKeyDown(Keys.Down))
                    ship_vector.Y += ship_speed.Y;
                if (keyboardState.IsKeyDown(Keys.Up))
                    ship_vector.Y -= ship_speed.Y;
            }

            // Kontrollera ifall rymdskeppet har kt  ut frn kanten, om det har
            // det, s terstll dess position.
            // Har det kt ut till vnster:
            if (ship_vector.X < 0)
                ship_vector.X = 0;
            // Har det kt ut till hger:
            if (ship_vector.X > Window.ClientBounds.Width - ship_texture.Width)
                ship_vector.X = Window.ClientBounds.Width - ship_texture.Width;
            // Har det kt ut upptill:
            if (ship_vector.Y < 0)
                ship_vector.Y = 0;
            // Har det kt ut nedtill:
            if (ship_vector.Y > Window.ClientBounds.Height - ship_texture.Height)
                ship_vector.Y = Window.ClientBounds.Height - ship_texture.Height;

            base.Update(gameTime);
        }

        // =======================================================================
        // Draw(), Hr ritas sjlva spelet ut.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // Rensa skrmen

            // Anvnd spriteBatch fr att rita ut saker p skrmen
            spriteBatch.Begin();
            spriteBatch.Draw(ship_texture, ship_vector, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
