﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace SpaceShooter
{
    // =======================================================================
    // Player, klass för att skapa ett spelarobjekt. Klassen ska hantera spel-
    // arens rymdskepp (sprite) och ta emot tangenttryckningar för att ändra
    // rymdskeppets position.
    // =======================================================================
    class Player
    {
        Texture2D texture; // Rymdskeppets textur
        Vector2 vector; // Rymdskeppets koordinater
        Vector2 speed; // Rymdskeppets hastighet

        // =======================================================================
        // Player(), konstruktor för att skapa spelar-objektet
        // =======================================================================
        public Player(Texture2D texture, float X, float Y, float speedX,
                      float speedY)
        {
            this.texture = texture;
            this.vector.X = X;
            this.vector.Y = Y;
            this.speed.X = speedX;
            this.speed.Y = speedY;
        }
        // =======================================================================
        // Update(), Tar emot tangenttryckningar och uppdaterar spelarens position
        // =======================================================================
        public void Update(GameWindow window)
        {

            // Läs in tangenttryckningar:
            KeyboardState keyboardState = Keyboard.GetState();

            // Flytta rymdskeppet efter tangenttryckningar (om det inte är på
            // väg ut från kanten):
            if (vector.X <= window.ClientBounds.Width - texture.Width
                && vector.X >= 0)
            {
                if (keyboardState.IsKeyDown(Keys.Right))
                    vector.X += speed.X;
                if (keyboardState.IsKeyDown(Keys.Left))
                    vector.X -= speed.X;
            }
            if (vector.Y <= window.ClientBounds.Height - texture.Height
                && vector.Y >= 0)
            {
                if (keyboardState.IsKeyDown(Keys.Down))
                    vector.Y += speed.Y;
                if (keyboardState.IsKeyDown(Keys.Up))
                    vector.Y -= speed.Y;
            }

            // Kontrollera ifall rymdskeppet har åkt  ut från kanten, om det har
            // det, så återställ dess position.
            // Har det åkt ut till vänster:
            if (vector.X < 0)
                vector.X = 0;
            // Har det åkt ut till höger:
            if (vector.X > window.ClientBounds.Width - texture.Width)
                vector.X = window.ClientBounds.Width - texture.Width;
            // Har det åkt ut upptill:
            if (vector.Y < 0)
                vector.Y = 0;
            // Har det åkt ut nedtill:
            if (vector.Y > window.ClientBounds.Height - texture.Height)
                vector.Y = window.ClientBounds.Height - texture.Height;

        }

        // =======================================================================
        // Draw(), ritar ut spelaren på skärmen
        // =======================================================================
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(texture, vector, Color.White);
        }
    }
}
