using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace SpaceShooter
{
    // =======================================================================
    // Game1, detta r vr huvudklass i spelet, det r hr all action sker!
    // =======================================================================
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics; // Anvnds fr att hantera grafik
        SpriteBatch spriteBatch; // Anvnds fr att rita bilder
        Player player; // Spelarens objekt
        PrintText printText; // Anvnds fr att rita ut saker p skrmen
        List<Enemy> enemies; // Lista med fiender

        // =======================================================================
        // Game1(), klassens konstruktor
        // =======================================================================
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        // =======================================================================
        // Initialize(), anropas d spelet startar. Hr ligger all kod fr att 
        // initiera objekt och skapa demdock inte laddning av olika filer (bilder,
        // ljud etc).
        // =======================================================================
        protected override void Initialize()
        {

            base.Initialize();
        }

        // =======================================================================
        // LoadContent(), anropas d spelet startar. Hr laddas alla objekt/filer
        // in (bilder, ljud etc)
        // =======================================================================
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Skapa spelaren:
            player = new Player(Content.Load<Texture2D>("images/player/ship"), 380, 400, 2.5f, 4.5f);
            
            // Skapa objekt fr att skriva ut text:
            printText = new PrintText(Content.Load<SpriteFont>("myFont"));
            
            // Skapa fiender
            enemies = new List<Enemy>();
            Random random = new Random();
            Texture2D tmpSprite = Content.Load<Texture2D>("images/enemies/mine");
            for (int i = 0; i < 10; i++)
            {
                int rndX = random.Next(0, Window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, Window.ClientBounds.Height / 2);
                Enemy temp = new Enemy(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lgg till i listan
            }

        }

        // =======================================================================
        // UnloadContent(), anropas d spelet avslutas. Hr kan man ladda ur de
        // objekt som skulle kunna behva det fr att rensa minne.
        // =======================================================================
        protected override void UnloadContent()
        {

        }

        // =======================================================================
        // Update(), frsta delen av spel-loopen. Hr frnras olika objekt. T.ex.
        // s lser vi in data frn anvndaren, flyttar p olika objekt osv.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Update(GameTime gameTime)
        {
            // Stnger av spelet om man trycker p Back-knappen p gamepaden.
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back ==
                                                              ButtonState.Pressed)
                this.Exit();

            player.Update(Window); // Uppdatera spelarens position

            // G igenom alla fiender
            foreach (Enemy e in enemies.ToList())
            {
                if (e.IsAlive) // Kontrollera om fienden lever
                    e.Update(Window); // Flytta p dem
                else // Ta bort fienden fr den r dd
                    enemies.Remove(e);
            }

            base.Update(gameTime);
        }

        // =======================================================================
        // Draw(), Hr ritas sjlva spelet ut.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // Rensa skrmen

            // Anvnd spriteBatch fr att rita ut saker p skrmen
            spriteBatch.Begin();
            
            player.Draw(spriteBatch); // Rita ut spelaren

            // Rita ut alla fiender:
            foreach (Enemy e in enemies)
                e.Draw(spriteBatch);

            printText.Print("antal fiender:" + enemies.Count, spriteBatch, 0, 0); // Rita ut testutskrift
            
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
