using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace SpaceShooter
{
    // =======================================================================
    // Game1, detta r vr huvudklass i spelet, det r hr all action sker!
    // =======================================================================
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics; // Anvnds fr att hantera grafik
        SpriteBatch spriteBatch; // Anvnds fr att rita bilder
        Player player; // Spelarens objekt
        PrintText printText; // Anvnds fr att rita ut saker p skrmen
        List<Enemy> enemies; // Lista med fiender
        List<GoldCoin> goldCoins; // Lista med guldmynt
        Texture2D goldCoinSprite; // Guldmyntens bild


        // =======================================================================
        // Game1(), klassens konstruktor
        // =======================================================================
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        // =======================================================================
        // Initialize(), anropas d spelet startar. Hr ligger all kod fr att 
        // initiera objekt och skapa demdock inte laddning av olika filer (bilder,
        // ljud etc).
        // =======================================================================
        protected override void Initialize()
        {
            goldCoins = new List<GoldCoin>();
            base.Initialize();
        }

        // =======================================================================
        // LoadContent(), anropas d spelet startar. Hr laddas alla objekt/filer
        // in (bilder, ljud etc)
        // =======================================================================
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Skapa spelaren:
            player = new Player(Content.Load<Texture2D>("images/player/ship"), 380, 400, 2.5f,
                    4.5f, Content.Load<Texture2D>("images/player/bullet"));

            
            // Skapa objekt fr att skriva ut text:
            printText = new PrintText(Content.Load<SpriteFont>("myFont"));
            
            // Skapa fiender
            enemies = new List<Enemy>();
            Random random = new Random();
            Texture2D tmpSprite = Content.Load<Texture2D>("images/enemies/mine");
            for (int i = 0; i < 5; i++)
            {
                int rndX = random.Next(0, Window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, Window.ClientBounds.Height / 2);
                Mine temp = new Mine(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lgg till i listan
            }
            tmpSprite = Content.Load<Texture2D>("images/enemies/tripod");
            for (int i = 0; i < 5; i++)
            {
                int rndX = random.Next(0, Window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, Window.ClientBounds.Height / 2);
                Tripod temp = new Tripod(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lgg till i listan
            }


            // Ladda in bild fr guldmynt:
            goldCoinSprite = Content.Load<Texture2D>("images/powerups/coin");
        }

        // =======================================================================
        // UnloadContent(), anropas d spelet avslutas. Hr kan man ladda ur de
        // objekt som skulle kunna behva det fr att rensa minne.
        // =======================================================================
        protected override void UnloadContent()
        {

        }

        // =======================================================================
        // Update(), frsta delen av spel-loopen. Hr frnras olika objekt. T.ex.
        // s lser vi in data frn anvndaren, flyttar p olika objekt osv.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Update(GameTime gameTime)
        {
            // Stnger av spelet om man trycker p Back-knappen p gamepaden.
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            player.Update(Window, gameTime); // Uppdatera spelarens position

            // G igenom alla fiender
            foreach (Enemy e in enemies.ToList())
            {
                // Kontrollera om fienden kolliderar med ett skott
                foreach (Bullet b in player.Bullets)
                {
                    if (e.CheckCollision(b)) // Kollision uppstod
                    {
                        e.IsAlive = false; // Dda fienden
                        player.Points++; // Ge spelaren pong
                    }
                }

                if (e.IsAlive) // Kontrollera om fienden lever
                {
                    // Kontrollera kollision med spelaren:
                    if (e.CheckCollision(player))
                        this.Exit();
                    e.Update(Window); // Flytta p dem
                }
                else // Ta bort fienden fr den r dd
                    enemies.Remove(e);
            }

            // Guldmynten ska uppst slumpmssigt, en chans p 200:
            Random random = new Random();
            int newCoin = random.Next(1, 200);
            if (newCoin == 1) // ok, nytt guldmynt ska uppst
            {
                // Var ska guldmyntet uppst:
                int rndX = random.Next(0, Window.ClientBounds.Width - goldCoinSprite.Width);
                int rndY = random.Next(0, Window.ClientBounds.Height - goldCoinSprite.Height);
                // Lgg till myntet i listan:
                goldCoins.Add(new GoldCoin(goldCoinSprite, rndX, rndY, gameTime));
            }

            // G igenom hela listan med existernade guldmynt
            foreach (GoldCoin gc in goldCoins.ToList())
            {
                if (gc.IsAlive) // Kontrollera om guldmyntet lever
                {
                    // gc.Update() kollar om guldmyntet har blivit fr gammalt
                    // fr att f leva vidare:
                    gc.Update(gameTime);

                    // Kontrollera om de kolliderat med spelaren:
                    if (gc.CheckCollision(player))
                    {
                        goldCoins.Remove(gc); // Ta bort myntet vid kollision...
                        player.Points++; // ...och ge spelaren pong
                    }
                }
                else // Ta bort guldmyntet fr det r dtt
                    goldCoins.Remove(gc);
            }

            base.Update(gameTime);
        }

        // =======================================================================
        // Draw(), Hr ritas sjlva spelet ut.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // Rensa skrmen

            // Anvnd spriteBatch fr att rita ut saker p skrmen
            spriteBatch.Begin();
            
            player.Draw(spriteBatch); // Rita ut spelaren

            // Rita ut alla fiender:
            foreach (Enemy e in enemies)
                e.Draw(spriteBatch);

            // Rita ut alla guldmynt:
            foreach (GoldCoin gc in goldCoins)
                gc.Draw(spriteBatch);

            // Skriv hur mnga pong spelaren har:
            printText.Print("points:" + player.Points, spriteBatch, 0, 0);
            
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
