using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace SpaceShooter
{
    // =======================================================================
    // Game1, detta r vr huvudklass i spelet.
    // =======================================================================
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics; // Anvnds fr att hantera grafik
        SpriteBatch spriteBatch; // Anvnds fr att rita bilder
        // =======================================================================
        // Game1(), klassens konstruktor
        // =======================================================================
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        // =======================================================================
        // Initialize(), anropas d spelet startar. Hr ligger all kod fr att 
        // initiera objekt och skapa dem dock inte laddning av olika filer
        // (bilder, ljud etc).
        // =======================================================================
        protected override void Initialize()
        {
            GameElements.currentState = GameElements.State.Menu;
            GameElements.Initialize();
            base.Initialize();
        }

        // =======================================================================
        // LoadContent(), anropas d spelet startar. Hr laddas alla objekt/filer
        // in (bilder, ljud etc)
        // =======================================================================
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            GameElements.LoadContent(Content, Window);
        }

        // =======================================================================
        // UnloadContent(), anropas d spelet avslutas. Hr kan man ladda ur de
        // objekt som skulle kunna behva det fr att rensa minne.
        // =======================================================================
        protected override void UnloadContent()
        {

        }

        // =======================================================================
        // Update(), frsta delen av spel-loopen. Hr frnras olika objekt. T.ex.
        // s lser vi in data frn anvndaren, flyttar p olika objekt osv.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Update(GameTime gameTime)
        {
            // Stnger av spelet om man trycker p Back-knappen p gamepaden.
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            switch (GameElements.currentState)
            {
                case GameElements.State.Run: // Kr sjlva spelet
                    GameElements.currentState = GameElements.RunUpdate(Content, Window,
                                                                gameTime);
                    break;
                case GameElements.State.HighScore: // Highscorelistan
                    GameElements.currentState = GameElements.HighScoreUpdate();
                    break;
                case GameElements.State.Quit: // Avsluta spelet
                    this.Exit();
                    break;
                default: // Menyn
                    GameElements.currentState = GameElements.MenuUpdate(gameTime);
                    break;
            }

            base.Update(gameTime);
        }

        // =======================================================================
        // Draw(), Hr ritas sjlva spelet ut.
        // gameTime, anvnds fr att hlla koll p spelets uppdateringsfrekvens.
        // =======================================================================
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // Rensa skrmen

            // Anvnd spriteBatch fr att rita ut saker p skrmen
            spriteBatch.Begin();

            switch (GameElements.currentState)
            {
                case GameElements.State.Run: // Kr sjlva spelet
                    GameElements.RunDraw(spriteBatch);
                    break;
                case GameElements.State.HighScore: // Highscorelistan
                    GameElements.HighScoreDraw(spriteBatch);
                    break;
                case GameElements.State.Quit: // Avsluta spelet
                    this.Exit();
                    break;
                default: // Menyn
                    GameElements.MenuDraw(spriteBatch);
                    break;
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }

}
