using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


// =======================================================================
// HsItem, en behllare-klass som innehller info om en person i
// highscorelistan.
// =======================================================================
class HSItem
{
	// Variabler och egenskaper fr dem:
	string name;
	int points;

	public string Name{ get { return name; } set { name = value; } }

	public int Points { get { return points; } set { points = value; } }

	// =======================================================================
	// HSItem(), klassens konstruktor
	// =======================================================================
	public HSItem(string name, int points)
	{
		this.name = name;
		this.points = points;
	}
}

// =======================================================================
// HighScore, innehller en lista med hsItems samt metoder fr att
// manipulera listan.
// =======================================================================
class HighScore
{
	int maxInList = 5; // Hur mnga som fr vara i listan
	List<HSItem> highscore = new List<HSItem>();
	string name; // Spelarens namn

	// Anvnds fr att skriva ut vilket tecken spelaren har valt just nu:
	string currentChar;
	int key_index = 0; // Denna anvnds fr att mata in spelarens namn
	// Dessa anvnds fr att kontrollera nr tangenter trycktes in:
	double lastChange = 0;
	Keys previousKey;

	// =======================================================================
	// HighScore(), klassens konstruktor
	// =======================================================================
	public HighScore(int maxInList)
	{
		this.maxInList = maxInList;
	}

	// =======================================================================
	// Sort(),  metod som sorterar listan. Metoden
	// anropas av Add() nr en ny person lggs till i
	// listan. Anvnder algoritmen bubblesort
	// =======================================================================
	void Sort()
	{
		int max = highscore.Count - 1;

		// Den yttre loopen, gr igenom hela listan            
		for (int i = 0; i < max; i++)
		{
			// Den inre, gr igenom element fr element
			int nrLeft = max - i; // Fr att se hur mnga som redan gtts igenom
			for (int j = 0; j < nrLeft; j++)
			{
				if (highscore[j].Points < highscore[j + 1].Points) // Jmfr elementen
				{
					// Byt plats!
					HSItem temp = highscore[j];
					highscore[j] = highscore[j + 1];
					highscore[j + 1] = temp;
				}
			}
		}
	}

	// =======================================================================
	// Add(), lgger till en person i highscore-listan.
	// =======================================================================
	void Add(int points)
	{
		// Skapa en temporr variabel av typen HSItem:
		HSItem temp = new HSItem(name, points);
		// Lgg till tmp i listan. Observera att fljande Add()
		// tillhr klassen List (r allts skapad av Microsoft).
		// Metoden har endast samma namn, som just denna Add():
		highscore.Add(temp);
		Sort(); // Sortera listan efter att vi har lagt till en person!

		// r det fr mnga i listan?
		if (highscore.Count > maxInList)
		{
			// Eftersom vi har lagt till endast en person nu, s betyder
			// det att det r en person fr mycket. Index p personen
			// som r sist i listan, r samma som maxInList. Vi vill ju
			// att det hgsta indexet ska vara maxInList-1. Allst kan
			// vi bara ta bort elementet med index maxInList.
			// Exempel:
			// maxInList r 5, vi har 6 element i listan. Det sjtte
			// elementet har index 5. Vi gr highscore.RemoveAt(5):
			highscore.RemoveAt(maxInList);
		}
	}

	// =======================================================================
	// CheckKey(), kontrollerar om en viss tangent har tryckts och huruvida
	// det har gtt lagomt lng tid (130ms) sedan tidigare tryck av samma
	// tangent.
	// =======================================================================
	bool CheckKey(Keys key, GameTime gameTime)
	{
		KeyboardState keyboardState = Keyboard.GetState();
		if (keyboardState.IsKeyDown(key))
		{
			// Har det gtt lagomt lng tid, eller r det en helt annan
			// tangent som trycks ned denna gng?
			if (lastChange + 130 < gameTime.TotalGameTime.TotalMilliseconds
				|| previousKey != key)
			{
				// stt om variablerna infr nsta varv i spelloopen:
				previousKey = key;
				lastChange = gameTime.TotalGameTime.TotalMilliseconds;
				return true;
			}
		}
		// Just den tangenten (key) trycktes INTE ned, eller s trycktes den
		// ned alldeles nyligen (mindre n 130ms):
		return false;
	}

	// =======================================================================
	// PrintDraw(), metod fr att skriva ut listan. Det finns ingen
	// PrintUpdate() d det r en helt statisk text som skrivs ut.
	// =======================================================================
	public void PrintDraw(SpriteBatch spriteBatch, SpriteFont font)
	{
		string text = "HIGHSCORE\n";
		foreach(HSItem h in highscore)
			text += h.Name + " " + h.Points + "\n";

		spriteBatch.DrawString(font, text, Vector2.Zero, Color.White);
	}

	// =======================================================================
	// EnterUpdate(), hr matar anvndaren in sitt anvndarnamn. Precis som
	// klassiska gamla arkadspel kan man ha tre tecken A-Z i sitt namn. Detta
	// r Update-delen i spel-loopen fr inmatning av highscore-namn. Metoden
	// ska fortstta anropas av Update() s lnge true returneras.
	// =======================================================================
	public bool EnterUpdate(GameTime gameTime, int points)
	{
		// Vilka tecken som r mjliga:
		char[] key = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K',
					   'L', 'M', 'N', 'O', 'P',  'Q', 'R', 'S', 'T', 'U',
					   'V', 'X', 'Y', 'Z'};


		// Anvndaren trycker knappen nedt, stega framlnges i key-vektorn:
		if (CheckKey(Keys.Down, gameTime))
		{
			key_index++;
			if (key_index >= key.Length)
				key_index = 0;
		}

		// Anvndaren trycker knappen uppt, stega baklnges i key-vektorn:
		if (CheckKey(Keys.Up, gameTime))
		{
			key_index--;
			if (key_index <= 0)
				key_index = key.Length - 1;
		}

		// Anvndaren trycker ENTER, lgg till det valda tecknet i 
		if (CheckKey(Keys.Enter, gameTime))
		{
			name += key[key_index].ToString();
			if (name.Length == 3)
			{
				// terstll namnet och allt s att man kan lgga till namnet 
				// p en ny spelare:
				Add(points);
				name = "";
				currentChar = "";
				key_index = 0;
				return true; // Ange att vi r klara
			}
		}
		// Lagra det tecken som nu r valt, s att vi kan skriva ut det i
		// EnterDraw():
		currentChar = key[key_index].ToString();
		// Ange att vi inte r klara, fortstt anropa denna metod via Update():
		return false;
	}

	// =======================================================================
	// EnterDraw(), skriver ut de tecken spelaren har matat in av sitt namn
	// (om ngot) samt det tecken (av tre) som just nu r valt.
	// =======================================================================
	public void EnterDraw(SpriteBatch spriteBatch, SpriteFont font)
	{
		string text = "ENTER NAME:" + name + currentChar;
		spriteBatch.DrawString(font, text, Vector2.Zero, Color.White);
	}

	// =======================================================================
	// SaveToFile(), spara till fil.
	// =======================================================================
	public void SaveToFile(string filename)
	{
	}

	// =======================================================================
	// LoadFromFile(), ladda frn fil.
	// =======================================================================
	public void LoadFromFile(string filename)
	{
	}
}

